@@ -103,8 +103,14 @@ class InvalidECGPattern(KestrelException):
     pass
 
 
+class MissingDataSource(KestrelException):
+    def __init__(self, stmt):
+        super().__init__(f"missing datasource in statement: {str(stmt)}")
+
+
 class VariableNotExist(KestrelException):
     def __init__(self, var_name):
+        self.var_name = var_name
         super().__init__(
             f'variable "{var_name}" does not exist', "check the variable used"
         )
@@ -118,11 +124,6 @@ class UnsupportedRelation(KestrelException):
         )
 
 
-class UnsupportedStixSyntax(KestrelException):
-    def __init__(self, msg):
-        super().__init__(msg, "rewrite the STIX pattern")
-
-
 ################################################################
 #                 Kestrel Code Generation Errors
 ################################################################
